package com.lrm.android.kurs2;

import android.content.Context;
import android.hardware.Sensor;
import android.hardware.SensorEvent;
import android.hardware.SensorEventListener;
import android.hardware.SensorManager;
import android.os.Environment;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.Button;
import android.widget.TextView;

import java.io.File;
import java.io.FileOutputStream;
import java.io.PrintStream;
import java.util.Locale;

public class MainActivity extends AppCompatActivity implements SensorEventListener {

    private SensorManager sensorManager;
    private Sensor accSensor;

    private PrintStream dataOutStream = null;
    private long startTimestamp = 0;
    private long curTimestamp;

    private int stepCount = 0;

    /**
     * Funkcja wywoływana zawsze podczas uruchamiania aplikacji.
     */
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // SensorManager jest klasą zarządzającą czujnikami.
        // Pobieramy menadżer odpowiedni dla kontekstu naszej aplikacji.
        sensorManager = (SensorManager)getSystemService(Context.SENSOR_SERVICE);
        //Pobieramy obiekt reprezentujący czujnik przyspieszenia.
        accSensor = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_settings) {
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    /**
     * Funkcja wywoływana zawsze podczas wznawiania działania aplikacji,
     * a także podczas jej uruchamiania.
     */
    @Override
    protected void onResume(){
        //Wywołanie funkcji onResume() klasy nadrzędnej.
        super.onResume();
        //Zgłoszenie funkcji reagujących na wydarzenia związane z sensorem accSensor.
        //Wartość SensorManager.SENSOR_DELAY_NORMAL oznacza domyślną częstotliwość
        //napływania danych - można eksperymentować z innymi (również z wartościami
        //liczbowymi).
        sensorManager.registerListener(this, accSensor, SensorManager.SENSOR_DELAY_NORMAL);
    }

    /**
     * Funkcja wywoływana zawsze podczas zatrzymywania aplikacji,
     */
    @Override
    protected void onPause(){
        //Wywołanie funkcji onPause() klasy nadrzędnej.
        super.onPause();
        //Zwolnienie funkcji reagujących na wydarzenie związane z sensorami.
        sensorManager.unregisterListener(this);

        //Wyłączenie nagrywania.
        Button recBtn = (Button) findViewById(R.id.recButton);
        stopRec(recBtn);
    }

    /**
     * Funkcja wywoływana gdy nadejdą nowe dane z sensorów. Może obsługiwać
     * wiele sensorów.
     * @param event Obiekt reprezentujący wydarzenie związane z sensorem.
     */
    @Override
    public void onSensorChanged(SensorEvent event){
        //Pobranie obiektu, który reprezentuje sensor, którego obecne zdarzenie.
        Sensor curSensor = event.sensor;

        //Jeśli sensor jest akcelerometrem.
        if(curSensor.getType() == Sensor.TYPE_ACCELEROMETER){
            //Pobranie wartości przyspieszeń we wszystkich 3 osiach.
            float x = event.values[0];
            float y = event.values[1];
            float z = event.values[2];

            //Wyświelenie wartośći przyspieszenie w polach tekstowych.
            TextView accXTextView = (TextView)findViewById(R.id.accXTextView);
            accXTextView.setText(String.valueOf(x));

            TextView accYTextView = (TextView)findViewById(R.id.accYTextView);
            accYTextView.setText(String.valueOf(y));

            TextView accZTextView = (TextView)findViewById(R.id.accZTextView);
            accZTextView.setText(String.valueOf(z));

            //Jeśli strumień do zapisu danych jest utworzony.
            if(dataOutStream != null){
                //Jeśli jeszcze nie ustawiono czasu rozpoczęcia pomiarów (jest to
                //pierwszy pomiar).
                if(startTimestamp == 0){
                    startTimestamp = event.timestamp;
                }
                curTimestamp = event.timestamp;

                //Zapis do strumienia w postaci formatowanego napisu - składnia podobna
                //jak w języku C. Użyto amerykańskich ustawień m.in. separatora dziesiętnego,
                //aby uniknąć kłopotów z dalszym przetwarzaniem danych.
                dataOutStream.printf(Locale.US, "%d %f %f %f", curTimestamp - startTimestamp, x, y, z);
                dataOutStream.print(System.getProperty("line.separator"));
            }

            //Tutaj należy uzupełnić algorytm wykrywający kroki. W razie potrzeby można tworzyć
            //dodatkowe zmienne wewnątrz klasy.



            //Koniec.

            //Wyświetlanie zmierzonej liczby kroków.
            TextView stpCntTextView = (TextView)findViewById(R.id.stpCntTextView);
            stpCntTextView.setText(String.valueOf(stepCount));
        }
    }

    /**
     * Funkcja wywoływana, gdy zmienią się ustawienia dokładności sensora. Nie używana
     * w naszej aplikacji.
     * @param sensor Obiekt reprezentujący sensor.
     * @param accuracy Nowa wartość dokładności.
     */
    @Override
    public void onAccuracyChanged(Sensor sensor, int accuracy){

    }

    /**
     * Funkcja rozpoczynająca nagrywanie danych.
     * @param btn Obiekt reprezentujący przycisk, który służy do włączania i zatrzymywania nagrywania.
     */
    private void startRec(Button btn){
        //Sprawdzanie statusu pamięci. Jeśli zwrócona wartość to "mounted" to wszystko jest w porządku.
        Log.d("kurs2", "external storage state: " + Environment.getExternalStorageState());

        //Sprawdzamy czy katalog "kurs2" istnieje za pomocą klasy File.
        File dir = new File(Environment.getExternalStorageDirectory().toString() +
                "/kurs2");
        //Jeśli nie istnieje to tworzymy go i wszystkie katalogi nadrzędne.
        if(!dir.exists()){
            dir.mkdirs();
        }

        //Ścieżka do pliku, do którego będą zapisywane dane.
        String dataOutPath = String.format(Locale.getDefault(),
                Environment.getExternalStorageDirectory().toString() +
                        "/kurs2/data.log");

        Log.d("kurs2", "dataOutPath: " + dataOutPath);

        //Java wymusza obsługę wszystkich potencjalnych wyjątków. Poniższa składnia służy
        //do wyłapywania wyjątków.
        try {
            //Otwieranie strumienia, do którego będą zapisywane dane.
            dataOutStream = new PrintStream(new FileOutputStream(dataOutPath));
        }
        //Jeśli wystąpił wyjątek.
        catch(Exception e){
            dataOutStream = null;
            btn.setText("Nagrywaj");

            //Wyświetlanie całej hierarchii wywołań - ułatwia wyszukanie miejsca wystąpenia wyjątku.
            e.printStackTrace();

            Log.d("kurs2", "Nie udało się otworzyć pliku");
        }
        //Jeśli udało się otworzyć strumień to zmień tekst na przycisku.
        if(dataOutStream != null) {
            btn.setText("Stop");
        }
    }

    /**
     * Funkcja zatrzymująca nagrywanie danych.
     * @param btn Obiekt reprezentujący przycisk, który służy do włączania i zatrzymywania nagrywania.
     */
    private void stopRec(Button btn){
        if(dataOutStream != null) {
            dataOutStream.close();
        }
        dataOutStream = null;
        startTimestamp = 0;
        btn.setText("Nagrywaj");
    }

    /**
     * Funkcja reagująca na naciśnięcie przycisku.
     * @param v Obiekt widoku, którym jest przycisk.
     */
    public void recButtonOnClick(View v){
        Button btn = (Button)v;

        if(dataOutStream == null){
            startRec(btn);
        }
        else{
            stopRec(btn);
        }
    }
}
